<?php
/**
 * $Horde: luxor/ident.php,v 1.26 2005/01/03 14:35:14 jan Exp $
 *
 * Copyright 2003-2005 Jan Schneider <jan@horde.org>
 * Copyright 2003-2005 Mike Cochrane <mike@graftonhall.co.nz>
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.horde.org/licenses/gpl.php.
 */

@define('LUXOR_BASE', dirname(__FILE__));
require_once LUXOR_BASE . '/lib/base.php';
require_once 'Horde/Template.php';

$source = $sources[$sourceid];
$index = &Luxor_Driver::singleton($sourceid);
$ident = Util::getFormData('i');

// Change source if the symbol isn't from the current source.
$symbolSource = $index->getSourceBySymbol($ident);
if ($symbolSource != $sourceid) {
    $source = $sources[$symbolSource];
    $index = &Luxor_Driver::singleton($symbolSource);
}

$declarations = $index->getIndex($ident);
$sorted = array();
foreach ($declarations as $decl) {
    $sorted[$decl['declaration']][] = array('filename' => $decl['filename'],
                                            'line' => $decl['line']);
}

$ds = array();
foreach ($sorted as $type => $locations) {
    $d = array();
    $d['title'] = sprintf(_("Declared as a %s"), $type);
    $d['files'] = array();

    usort($locations, create_function('$a, $b', 'return strcasecmp($a["filename"], $b["filename"]);'));
    foreach ($locations as $loc) {
        $link = Horde::link(Luxor::url('source', $loc['filename'], array(), $loc['line']), $loc['filename']);
        $d['files'][] = str_repeat('&nbsp;', 4) . $link . $loc['filename'] . ' ' . sprintf(_("Line %s"), $loc['line']) . '</a>';
    }
    $ds[] = $d;
}

$references = $index->getReference($ident);
usort($references, create_function('$a, $b', 'return strcasecmp($a["filename"], $b["filename"]);'));

$curfile = '';
$rs = array();
foreach ($references as $info) {
    if ($curfile != $info['filename']) {
        if (isset($r)) {
            $rs[] = $r;
        }

        $curfile = $info['filename'];

        $r = array();
        $r['file'] = Horde::link(Luxor::url('source', $info['filename']), $info['filename']) . $info['filename'] . '</a>';
        $r['lines'] = array();
    }

    $r['lines'][] = str_repeat('&nbsp;', 4) . Horde::link(Luxor::url('source', $info['filename'], array(), $info['line']), $info['line']) . sprintf(_("Line %s"), $info['line']) . '</a>';
}
$rs[] = $r;

$title = sprintf(_("%s :: Symbol '%s'"), $source['name'], $index->symname($ident));
require LUXOR_TEMPLATES . '/common-header.inc';
require LUXOR_TEMPLATES . '/menu.inc';

$template = &new Horde_Template();
$template->set('title', $title, true);
$template->set('declarations', $ds, true);
$template->set('references', $rs, true);
$template->setOption('gettext', true);
echo $template->fetch(LUXOR_TEMPLATES . '/ident/ident.html');

require $registry->get('templates', 'horde') . '/common-footer.inc';
