<?php
/**
 * $Horde: luxor/lib/Luxor.php,v 1.42 2004/11/22 02:43:01 chuck Exp $
 *
 * @author  Jan Schneider <jan@horde.org>
 * @version $Revision: 1.42 $
 * @since   Luxor 0.1
 * @package Luxor
 */
class Luxor {

    /**
     * Generate a URL that links into Luxor.
     *
     * @param string $script  Name of the Luxor script to link into
     * @param string $uri     Any PATH_INFO portion that should be included
     * @param array $args     Key/value pair of any GET parameters to append
     * @param string $anchor  Anchor entity name
     */
    function url($script, $uri = '', $args = array(), $anchor = '')
    {
        global $conf, $sourceid;

        $script = $script . '.php';
        $arglist = array_merge(array('source' => $sourceid), $args);

        if ($conf['options']['use_path_info']) {
            if (substr($uri, 0, 1) != '/') {
                $script .= '/';
            }
            $script .= $uri;
        } else {
            $arglist['f'] = $uri;
        }

        $url = Horde::applicationUrl($script);

        foreach ($arglist as $key => $val) {
            if (!empty($val) || $val === 0) {
                $url = Util::addParameter($url, $key, $val);
            }
        }

        if (!empty($anchor)) {
            $url .= "#$anchor";
        }

        return $url;
    }

    /**
     * Generate a list of sources available from this installation
     * of Luxor.
     *
     * @return XHTML code representing links to the repositories
     */
    function sources()
    {
        global $source, $sources;

        $arr = array();
        foreach ($sources as $key => $val) {
            if ($val != $source) {
                $arr[] = '<b>' . Horde::link(Luxor::url('source', '', array('source' => $key)), $val['name'], 'smallheader') .
                    $val['name'] . '</a></b>';
            }
        }

        if (count($arr)) {
            return _("Other Sources") . ': ' . implode(', ', $arr);
        } else {
            return '';
        }
    }

    /**
     * Sanitizes path names passed by the user.
     *
     * @param string $node  The path name to clean up.
     *
     * @return string       The cleaned up path.
     */
    function fixPaths($node)
    {
        global $files;

        $node = '/' . $node;
        $node = preg_replace('|/[^/]+/\.\./|', '/', $node);
        $node = preg_replace('|/\.\./|', '/', $node);
        if ($files->isDir($node)) {
            $node .= '/';
        }
        $node = preg_replace('|//+|', '/', $node);

        return $node;
    }

    /**
     *
     */
    function outfun($str, $arr)
    {
        return str_replace("\n", "\n" . array_shift($arr), $str);
    }

    function dirExpand($dir)
    {
        global $files, $mime_drivers, $mime_drivers_map;

        require_once 'Horde/MIME/Magic.php';
        require_once 'Horde/MIME/Viewer.php';
        require_once HORDE_BASE . '/config/mime_drivers.php';
        require_once LUXOR_BASE . '/config/mime_drivers.php';

        $nodes = $files->getDir($dir);
        if (is_a($nodes, 'PEAR_Error')) {
            return $nodes;
        }
        $index = $files->getIndex($dir);
        if (is_a($index, 'PEAR_Error')) {
            return $index;
        }

        if ($dir != '/') {
            array_unshift($nodes, '../');
        }

        $list = array();
        foreach ($nodes as $node) {
            $link = Horde::link(Luxor::url('source', $dir . $node), $node == '../' ? _("Up to parent") : $node);
            $modtime = $files->getFiletime($dir . $node);
            if (!$modtime) {
                $modtime = '-';
            } else {
                $modtime = gmdate('Y-m-d H:i:s', $modtime);
            }
            $description = empty($index[$node]) ? '&nbsp;' : $index[$node];
            if (substr($node, -1) == '/') {
                if ($node == '../') {
                    $icon = Horde::img('parent.png', _("Up to parent"));
                    $node = _("Parent Directory");
                } else {
                    $icon = Horde::img('folder.png', $node);
                }

                $list[] = array('iconlink'    => $link . $icon . '</a>',
                                'namelink'    => $link . $node . '</a>',
                                'filesize'    => '-',
                                'bytes'       => '',
                                'modtime'     => $modtime,
                                'bgcolor'     => '',
                                'css'         => '',
                                'description' => $description);
            } else {
                if (preg_match('/^.*\.[oa]$|^core$|^00-INDEX$/', $node)) {
                    continue;
                }
                $icon = Horde::img(MIME_Viewer::getIcon(MIME_Magic::filenameToMIME($node)), '', '', '');
                $filesize = $files->getFilesize($dir . $node);
                if ($filesize < 1 << 10) {
                    $bytes = _("bytes");
                } else {
                    $bytes = _("kbytes");
                    $filesize = $filesize >> 10;
                }
                $list[] = array('iconlink'    => $link . $icon . '</a>',
                                'namelink'    => $link . $node . '</a>',
                                'filesize'    => $filesize,
                                'bytes'       => $bytes,
                                'modtime'     => $modtime,
                                'bgcolor'     => '',
                                'css'         => '',
                                'description' => $description);
            }
        }

        return $list;
    }

    /**
     * Prints a descriptive blurb at the end of directory listings.
     *
     * @param Luxor_File $files  An instance of Luxor_File.
     * @param string $path       The directory where to look for a README file.
     */
    function dirDesc($files, $path)
    {
        $table_head = '<br /><br /><table width="100%" cellpadding="5"><tr><td class="text"><span class="fixed">';
        $table_foot = '</span></td></tr></table>';
        if (@file_exists($filename = $files->toReal($path . '/README')) ||
            @file_exists($filename = $files->toReal($path . '/README.txt'))) {
            require_once 'Horde/Text/Filter.php';

            $fp = fopen($filename, "r");
            $contents = fread($fp, filesize($filename));
            fclose($fp);

            return $table_head . Text_Filter::filter($contents, 'text2html', array('parselevel' => TEXT_HTML_MICRO)) . $table_foot;
        } elseif ($filename = @file_exists($files->toReal($path . '/README.html'))) {
            global $mime_drivers, $mime_drivers_map;
            require_once 'Horde/MIME/Part.php';
            require_once 'Horde/MIME/Viewer.php';
            require_once HORDE_BASE . '/config/mime_drivers.php';
            require_once LUXOR_BASE . '/config/mime_drivers.php';

            $fp = fopen($filename, "r");
            $contents = fread($fp, filesize($filename));
            fclose($fp);

            $mime_part = new Mime_Part('text/plain', $contents);
            $mime_part->setName('README');
            $viewer = MIME_Viewer::factory($mime_part);
            return $table_head . $viewer->render() . $table_foot;
        }
    }

    /**
     * Smaller version of the markupFile() function meant for marking up
     * the descriptions in source directory listings.
     *
     * @see markupFile()
     *
     * @todo most of this can be done by Text::toHtml()
     */
    function markupString($string, $virtp)
    {
		$string = htmlspecialchars($string);

        // HTMLify email addresses and urls.
        $string = preg_replace('#((ftp|http|nntp|snews|news)://(\w|\w\.\w|\~|\-|\/|\#)+(?!\.\b))#',
							   '<a href="$1">$1</a>', $string);

        // HTMLify file names, assuming file is in the current directory.
        $string = preg_replace('#\b(([\w\-_\/]+\.(c|h|cc|cp|hpp|cpp|java))|README)\b#e',
							   '"<a href=\" . Luxor::url("source", "' . $virtp . '$1") . "$1</a>"', $string);

        return $string;
    }

    function whereMenu()
    {
        global $pathname;

        $res = '';
        $wherePath = '';
        foreach (explode('/', $pathname) as $dir) {
            $wherePath .= "/$dir";
            if (!empty($dir)) {
                $res .= ' :: ' . Horde::link(Luxor::url('source', $wherePath), htmlspecialchars($dir), 'smallheader') .
					htmlspecialchars($dir) . '</a>';
            }
        }
        return $res;
    }

    function fileRef($desc, $css, $path, $line = 0, $args = array())
    {
        if ($line > 0 && strlen($line) < 3) {
            $line = str_repeat('0', (3 - strlen($line))) . $line;
        }

        return Horde::link(Luxor::url('source', $path, $args, $line > 0 ? $line : ''), $desc, $css) .
            $desc . '</a>';
    }

    function incRef($name, $css, $file, $paths = array())
    {
        global $files;

        foreach ($paths as $dir) {
            $dir = preg_replace('|/+$|', '', $dir);
            $path = $dir . '/' . $file;
            if ($files->isFile($path)) {
                return Luxor::fileRef($name, $css, $path);
            }
        }

        return $name;
    }

    /**
     * Check if the given item is restricted from being shown.
     *
     * @param string $filename  The filename to check
     *
     * @return boolean  Whether or not the item is allowed to be displayed
     **/
    function isRestricted($filename)
    {
        global $source;

        if (Auth::isAdmin()) {
            return false;
        }

        if (isset($source['restrictions']) && is_array($source['restrictions'])) {
            foreach ($source['restrictions'] as $restriction) {
                if (preg_match('|' . str_replace('|', '\|', $restriction) . '|', $filename)) {
                    return true;
                }
            }
        }

        return false;
    }

    /**
     * Check if the given directory is filtered out.
     *
     * @param string $dir  The path to check.
     *
     * @return boolean  True if the directory should be shown/parsed, false otherwise.
     **/
    function isDirParsed($dir)
    {
        global $source;

        if (isset($source['dirFilter']) && is_array($source['dirFilter'])) {
            foreach ($source['dirFilter'] as $filter) {
                if (preg_match('/' . str_replace('/', '\/', substr($filter, 1)) . '/', $dir)) {
                    return substr($filter, 0, 1) == '+' ? true : false;
                }
            }
        }

        if (isset($source['dirUnmatched'])) {
            return $source['dirUnmatched'];
        }
        return true;
    }

    /**
     * Check if the given file should be parsed an/or displayed.
     *
     * @param string $file  The filename to check.
     *
     * @return boolean  True if the file should be shown/parsed, false otherwise.
     **/
    function isFileParsed($file)
    {
        global $source;

        if (isset($source['fileFilter']) && is_array($source['fileFilter'])) {
            foreach ($source['fileFilter'] as $filter) {
                if (preg_match('/' . str_replace('/', '\/', substr($filter, 1)) . '/', $file, $matches)) {
                    return substr($filter, 0, 1) == '+' ? true : false;
                }
            }
        }

        if (isset($source['fileUnmatched'])) {
            return $source['fileUnmatched'];
        }
        return true;
    }

    /**
     * Pre and Postfix every line of a string with strings.
     */
    function fixString($string, $pre = '', $post = '')
    {
        require_once 'Horde/Text.php';
        $lines = preg_split('(\r\n|\n|\r)', $string);
        $res = '';
        foreach ($lines as $line) {
            $res .= !empty($res) ? "\n" : '';
            $res .= $pre . Text::htmlAllSpaces($line) . $post;
        }
        return $res;
    }

    function markupfile($pathname, $fileh, $ann = array())
    {
        global $files;

        preg_match_all('|^(.*/)|', $pathname, $dir);
        $dir = $dir[0];

        /* Determine the file's language and create a Luxor_Lang
		 * instance. */
        require_once LUXOR_BASE . '/lib/Lang.php';
        $lang = &Luxor_Lang::builder($files, $pathname);
        if (is_a($lang, 'PEAR_Error')) {
            return $lang;
        }

        $html = '<table border="0" cellpadding="0" cellspacing="0" width="100%" class="text">';

        // A source code file.
        if ($lang) {
            require_once LUXOR_BASE . '/lib/SimpleParse.php';
            $parser = &new Luxor_SimpleParse($fileh, 1, $lang->_langmap['spec']);
            $linenum = 1;

            list($btype, $frag) = $parser->nextFrag();
            $ofrag = '';
            while ($frag) {
                switch ($btype) {
                case 'comment':
                    // Comment
                    // Convert mail adresses to mailto:
                    // &freetextmarkup($frag);
                    // $lang->processComment(\$frag);
                    $frag = Luxor::fixString($frag, '<span class="comment">', '</span>');
                    break;

                case 'string':
                    $frag = Luxor::fixString($frag, '<span class="string">', '</span>');
                    break;

                case 'include':
                    // Include directive
                    $frag = $lang->processInclude($frag, $dir);
                    break;

                case 'variable':
                    $pre = sprintf('<span class="variable"><span class="var_%s" onmouseover="show_var(\'var_%s\');" onmouseout="unshow_var(\'var_%s\');">', substr($frag, 1), substr($frag, 1), substr($frag, 1));
                    $frag = Luxor::fixString($frag, $pre, '</span></span>');
                    break;

                default:
                    // Code
                    // somehow get $source['may_reference'] into the second parameter here.
                    $frag = $lang->processCode($frag);
                }

                $ofrag .= $frag;
                list($btype, $frag) = $parser->nextFrag();
            }
            $lines = preg_split('(\r\n|\n|\r)', $ofrag);
            foreach ($lines as $line) {
                $html .= '<tr><td align="right" style="padding-left:10px; padding-right:10px;"><a name="' . $linenum . '" class="fixed" style="color:black">' . $linenum++ . '</a></td><td width="100%" class="fixed">' . $line . "</td></tr>\n";
            }
            $html .= '</table>';
        } else {
            return false;
        }
        return $html;
    }

    /**
     * Build Luxor's list of menu items.
     *
     * @access public
     */
    function getMenu($returnType = 'object')
    {
        global $registry;

        require_once 'Horde/Menu.php';

        $menu = &new Menu(HORDE_MENU_MASK_ALL);
        $menu->add(Horde::applicationUrl('source.php'), _("Home"), 'luxor.png');

        if ($returnType == 'object') {
            return $menu;
        } else {
            return $menu->render();
        }
    }

}

/**
 * Luxor_Files:: defines an API to access source file repositories.
 *
 * @author  Jan Schneider <jan@horde.org>
 * @version $Revision: 1.42 $
 * @since   Luxor 0.1
 * @package Luxor
 */
class Luxor_Files {

    /**
     * Attempts to return a concrete Luxor_Files instance based on $driver.
     *
     * @param string    $driver     The type of concrete Luxor_Files subclass
     *                              to return.  The is based on the repository
     *                              driver ($driver).  The code is dynamically
     *                              included.
     * @param array     $params     (optional) A hash containing any additional
     *                              configuration or connection parameters a
     *                              subclass might need.
     *
     * @return mixed    The newly created concrete Luxor_Files instance, or
     *                  false on an error.
     */
    function &factory($driver, $params = array())
    {
        $driver = basename($driver);
        require_once dirname(__FILE__) . '/Files/' . $driver . '.php';
        $class = 'Luxor_Files_' . $driver;
        if (class_exists($class)) {
            return $ret = &new $class($params);
        } else {
            return false;
        }
    }

    /**
     * Attempts to return a reference to a concrete Luxor_Files instance
     * based on $driver.  It wil only create a new instance if no
     * Luxor_Files instance with the same parameters currently exists.
     *
     * This should be used if multiple source file repositories are required.
     *
     * This method must be invoked as: $var = &Luxor_Files::singleton()
     *
     * @param string    $driver     The type of concrete Luxor_Files subclass
     *                              to return.  The is based on the repository
     *                              driver ($driver).  The code is dynamically
     *                              included.
     * @param array     $params     (optional) A hash containing any additional
     *                              configuration or connection parameters a
     *                              subclass might need.
     *
     * @return mixed    The created concrete Luxor_Files instance, or false
     *                  on error.
     */
    function &singleton($driver, $params = array())
    {
        static $instances;

        if (!isset($instances)) {
            $instances = array();
        }

        $signature = serialize(array($driver, $params));
        if (!isset($instances[$signature])) {
            $instances[$signature] = &Luxor_Files::factory($driver, $params);
        }

        return $instances[$signature];
    }

}

/**
 * Luxor_Driver:: defines an API for implementing storage backends for Luxor.
 *
 * @author  Jan Schneider <jan@horde.org>
 * @version $Revision: 1.42 $
 * @since   Luxor 0.1
 * @package Luxor
 */
class Luxor_Driver {

    /**
     * Attempts to return a concrete Luxor_Driver instance based on $driver.
     *
     * @param string    $driver     The type of concrete Luxor_Driver subclass
     *                              to return.  The is based on the storage
     *                              driver ($driver).  The code is dynamically
     *                              included.
     *
     * @param array     $params     (optional) A hash containing any additional
     *                              configuration or connection parameters a
     *                              subclass might need.
     *
     * @return mixed    The newly created concrete Luxor_Driver instance, or
     *                  false on an error.
     */
    function &factory($source, $driver = null, $params = null)
    {
        if (is_null($driver)) {
            $driver = $GLOBALS['conf']['storage']['driver'];
        }

        $driver = basename($driver);

        if (is_null($params)) {
            $params = Horde::getDriverConfig('storage', $driver);
        }

        require_once dirname(__FILE__) . '/Driver/' . $driver . '.php';
        $class = 'Luxor_Driver_' . $driver;
        if (class_exists($class)) {
            return $ret = &new $class($source, $params);
        } else {
            return false;
        }
    }

    /**
     * Attempts to return a reference to a concrete Luxor_Driver instance
     * based on $driver.  It wil only create a new instance if no
     * Luxor_Driver instance with the same parameters currently exists.
     *
     * This should be used if multiple storage sources are required.
     *
     * This method must be invoked as: $var = &Luxor_Driver::singleton()
     *
     * @param string    $driver     The type of concrete Luxor_Driver subclass
     *                              to return.  The is based on the storage
     *                              driver ($driver).  The code is dynamically
     *                              included.
     *
     * @param array     $params     (optional) A hash containing any additional
     *                              configuration or connection parameters a
     *                              subclass might need.
     *
     * @return mixed    The created concrete Luxor_Driver instance, or false
     *                  on error.
     */
    function &singleton($source, $driver = null, $params = null)
    {
        static $instances;

        if (is_null($driver)) {
            $driver = $GLOBALS['conf']['storage']['driver'];
        }

        if (is_null($params)) {
            $params = Horde::getDriverConfig('storage', $driver);
        }

        if (!isset($instances)) {
            $instances = array();
        }

        $signature = serialize(array($source, $driver, $params));
        if (!isset($instances[$signature])) {
            $instances[$signature] = &Luxor_Driver::factory($source, $driver, $params);
        }

        return $instances[$signature];
    }

}
